% This function produces a video depicting the convergence of sequential
% Fourier Series approximations of the parametric equation (X(t), Y(t)).
% X: an input vector representing X(t)
% Y: an input vector representing Y(t)
% aviFilename: filename (String) of the resultant video to be saved
% peridoicity: a boolean marker indicating whether (X(t), Y(t)) is a
% periodic function. If the function is not periodic, the operation is
% performed on (X(t), Y(t)) followed by its mirror, and assumes periodicity

function  ParametricFourierSeriesConvergenceVid(X, Y, aviFilename, periodicity)
    
    %% Input Validation
    validateattributes(X, {'numeric'}, {'vector'}, mfilename, 'X', 1);                               % ensure X is a vector
    validateattributes(Y, {'numeric'}, {'vector', 'numel', length(X)}, mfilename, 'Y', 2);           % ensure Y is a vector of same length
    validateattributes(aviFilename, {'char'}, {'vector'}, mfilename, 'aviFilename', 3);              % ensure filename is a string
    validateattributes(periodicity, {'numeric'}, {'scalar', 'binary'}, mfilename, 'periodicity', 4); % ensure periodicity is boolean
 
    %% Set-Up Operations
    % add '.avi' to the end of the filename if necessary
    if (~endsWith(aviFilename, '.avi'))           
        aviFilename = [aviFilename, '.avi'];
    end
    
    % ensure we are dealing with column vectors
    [r, c] = size(X);
    if (r == 1)
        X = X';
    end
    
    [r, c] = size(Y);
    if (r == 1)
        Y = Y';
    end
    
    % this operation ensures that we are working with a periodic function
    if (~periodicity)               
        X = [X; flipud(X)];
        Y = [Y; flipud(Y)];
    end
    
    % open up the video to write
    outputVideo = VideoWriter(fullfile(aviFilename));     
    outputVideo.FrameRate = 3;    % 3 frames / s
    open(outputVideo);
    
    % create a directory to store graphs
    workingDir = 'tempDir';             
    mkdir(workingDir);
 
    % Determine graph bounds, create invisible figure
    xBounds = [min(X) - 0.1 * (max(X) - min(X)), max(X) + 0.1 * (max(X) - min(X))];  
    yBounds = [min(Y) - 0.1 * (max(Y) - min(Y)), max(Y) + 0.1 * (max(Y) - min(Y))];
    fig = figure('visible', 'off');   % create an invisible figure
    
    %% Calculations
    [Xa_k, N] = DTFS(X);    % compute a_1 through a_N Fourier Series coefficients for X,
    Ya_k = DTFS(Y);         % and for Y

    % Build reconstructed signal
    Xr = repmat(Xa_k(N), 1, N);         % X(t) reconstructed, starting with a_0
    Yr = repmat(Ya_k(N), 1, N);         % Y(t) ""
    makeGraph(0);                       % graph this (point) signal
    
    % Determine which graphs to keep (we don't want too many)
    keep = [1:50, 55:5:100, 125:25:1000, 1100:100:10000, 12500 : 2500 100000];
    
    n = 1 : N;                          % vector from 1 to N
    for k = 1 : floor((N - 1) / 2)      % keep adding a_-k and a_k and graphing
        Xr = Xr + Xa_k(k) * exp(1j * (k * n) * 2 * pi / N) + Xa_k(N - k) * exp(1j * ((N - k) * n) * 2 * pi / N);
        Yr = Yr + Ya_k(k) * exp(1j * (k * n) * 2 * pi / N) + Ya_k(N - k) * exp(1j * ((N - k) * n) * 2 * pi / N);  
        if (ismember(k, keep))
            makeGraph(k);
        end
    end
    if (mod(N, 2) == 2)                 % maybe need to add last term if even, then graph
        Xr = Xr + Xa_k(N / 2) * exp(1j * (N / 2 * n) * 2 * pi / N); 
        Yr = Yr + Ya_k(N / 2) * exp(1j * (N / 2 * n) * 2 * pi / N);
    end
    makeGraph(N);

    %% Video Generation
    % get all graph images from the created directory
    imageNames = dir(fullfile(workingDir, '*.jpg'));
    imageNames = {imageNames.name}';

    % write images in sequence to the outpur video, then delete them
    for k = 1 : length(imageNames)
        img = imread(fullfile(workingDir, imageNames{k}));
        writeVideo(outputVideo, img);
        delete(fullfile(workingDir, imageNames{k}));
    end

    close(outputVideo);
    rmdir(workingDir);
    
    %% Helper Functions
    function [a_k, N] = DTFS(X)     % returns a_1 through a_N, and N fro a signal
        [r, c] = size(X);
        if (r ~= 1 && c ~= 1)
            error('DTFS expects a vector as an argument');
        end
        if (r == 1)
            X = X';
        end
        N = length(X);
        n_f = (1 : N)';
        a_k = zeros(1, N);
        for k_f = 1 : N
            a_k(k_f) = 1 / N * sum(X .* exp(-1j * k_f * 2 * pi / N * n_f));
        end
    end

    function makeGraph(num)         % creates a graph of (X(t), Y(t)) and saves it
        if (num == 0)
            plot(real(Xr), real(Yr), 'k.');
        elseif (periodicity)
            plot(real([Xr Xr(1)]), real([Yr Yr(1)]), 'k-');
        else
            plot(real(Xr), real(Yr), 'k-');
        end
        xlim(xBounds);
        ylim(yBounds);
        if (num == N)
            title(sprintf('Fourier Series Approximation of (x(t), y(t)) with %5d term(s)', N));
        else
            title(sprintf('Fourier Series Approximation of (x(t), y(t)) with %5d term(s)', 1 + num * 2));
        end
        filename = sprintf('img%0*d.jpg', ceil(log10(N)) + 1, num);
        fullname = fullfile(workingDir, filename);  
        saveas(fig, fullname);
    end

end